@@ -29,18 +29,17 @@
 
 #include <glib.h>
 #include <glib/gi18n.h>
-#include <gconf/gconf-client.h>
 
 #include "gpm-common.h"
 #include "gpm-hal.h"
-#include "gpm-hal-monitor.h"
-#include "gpm-hal-power.h"
+#include "gpm-battery.h"
 
 #include "gpm-power.h"
 #include "gpm-marshal.h"
 #include "gpm-refcount.h"
+#include "gpm-ac-adapter.h"
 #include "gpm-debug.h"
-#include "gpm-gconf.h"
+#include "gpm-conf.h"
 
 static void     gpm_power_class_init (GpmPowerClass *klass);
 static void     gpm_power_init       (GpmPower      *power);
@@ -52,37 +51,28 @@ static void     gpm_power_finalize   (GObject       *object);
 
 struct GpmPowerPrivate
 {
-	gboolean		 on_ac;
-	gint			 exp_ave_factor;
+	guint			 exp_ave_factor;
 	gboolean		 data_is_trusted;
 	GpmRefcount		*refcount;
 	GHashTable		*battery_kind_cache;
 	GHashTable		*battery_device_cache;
+	GpmBattery		*battery;
+	GpmAcAdapter		*ac_adapter;
+	GpmConf			*conf;
 	GpmHal			*hal;
-	GpmHalMonitor		*hal_monitor;
-	GpmHalPower		*hal_power;
 };
 
 enum {
-	BUTTON_PRESSED,
-	AC_STATE_CHANGED,
 	BATTERY_STATUS_CHANGED,
-	BATTERY_REMOVED,
 	BATTERY_PERHAPS_RECALL,
 	LAST_SIGNAL
 };
 
-enum {
-	PROP_0,
-	PROP_ON_AC
-};
-
 static guint	     signals [LAST_SIGNAL] = { 0, };
-static gpointer      gpm_power_object = NULL;
 
 G_DEFINE_TYPE (GpmPower, gpm_power, G_TYPE_OBJECT)
 
-#define GPM_POWER_INVALID_TIMOUT 3000
+#define GPM_POWER_INVALID_TIMOUT 500
 
 /**
  * Multiple batteries percentages are averaged and times added
@@ -203,13 +193,11 @@ battery_device_cache_entry_update_all (GpmPower *power, GpmPowerDevice *entry)
 	gchar *udi = entry->udi;
 	gchar *battery_kind_str;
 	gboolean perhaps_recall;
+	gboolean show_recall;
 
 	g_return_val_if_fail (power != NULL, FALSE);
 	g_return_val_if_fail (GPM_IS_POWER (power), FALSE);
 
-	/* invalidate last rate */
-	entry->charge_rate_previous = 0;
-
 	/* Initialize battery_status to reasonable defaults */
 	gpm_power_battery_status_set_defaults (status);
 
@@ -318,14 +306,20 @@ battery_device_cache_entry_update_all (GpmPower *power, GpmPowerDevice *entry)
 	gpm_hal_device_get_string (power->priv->hal, udi, "battery.model", &entry->model);
 	gpm_hal_device_get_uint (power->priv->hal, udi, "battery.voltage.current", &status->voltage);
 
-	/* this is more common than you might expect */
-	gpm_hal_device_get_bool (power->priv->hal, udi, "info.perhaps_recalled", &perhaps_recall);
-	if (perhaps_recall) {
+	/* this is more common than you might expect: hardware that might blow up */
+	gpm_hal_device_get_bool (power->priv->hal, udi, "info.is_recalled", &perhaps_recall);
+
+	/* do we show the notification? */
+	gpm_conf_get_bool (power->priv->conf, GPM_CONF_SHOW_BATTERY_WARNING, &show_recall);
+
+	if (perhaps_recall == TRUE && show_recall == TRUE) {
 		gchar *oem_vendor;
 		gchar *website;
-		gpm_hal_device_get_string (power->priv->hal, udi, "info.recall.oem_url_link_text", &oem_vendor);
-		gpm_hal_device_get_string (power->priv->hal, udi, "info.recall.oem_url_link_target", &website);
+		gpm_hal_device_get_string (power->priv->hal, udi, "info.recall.vendor", &oem_vendor);
+		gpm_hal_device_get_string (power->priv->hal, udi, "info.recall.website_url", &website);
 		g_signal_emit (power, signals [BATTERY_PERHAPS_RECALL], 0, oem_vendor, website);
+		g_free (oem_vendor);
+		g_free (website);
 	}
 
 	if (entry->battery_kind == GPM_POWER_KIND_PRIMARY) {
@@ -352,7 +346,7 @@ battery_device_cache_entry_update_all (GpmPower *power, GpmPowerDevice *entry)
  * does not change drastically between updates.
  **/
 static int
-gpm_power_exp_aver (gint previous, gint new, gint factor_pc)
+gpm_power_exp_aver (gint previous, gint new, guint factor_pc)
 {
 	gint result = 0;
 	gfloat factor = 0;
@@ -416,12 +410,6 @@ battery_device_cache_entry_update_key (GpmPower	      *power,
 	} else if (strcmp (key, "battery.charge_level.rate") == 0) {
 		gpm_hal_device_get_uint (power->priv->hal, udi, key, &status->charge_rate_raw);
 
-		/* Do an exponentially weighted average, fixes bug #328927 */
-		status->charge_rate_smoothed = gpm_power_exp_aver (entry->charge_rate_previous,
-							status->charge_rate_raw,
-							power->priv->exp_ave_factor);
-		entry->charge_rate_previous = status->charge_rate_smoothed;
-
 	} else if (strcmp (key, "battery.charge_level.percentage") == 0) {
 		gpm_hal_device_get_uint (power->priv->hal, udi, key, &status->percentage_charge);
 
@@ -744,7 +732,7 @@ gpm_power_status_for_device (GpmPowerDevice *device)
 		g_string_append_printf (details, "<b>%s</b> %s\n",
 					_("Status:"), _("Discharging"));
 	}
-	if (status->percentage_charge > 0) {
+	if (status->percentage_charge >= 0) {
 		g_string_append_printf (details, "<b>%s</b> %i%%\n",
 					_("Percentage charge:"), 
 					status->percentage_charge);
@@ -939,14 +927,10 @@ gpm_power_get_icon_for_all (GpmPowerStatus *device_status,
 		index_str = gpm_power_get_index_from_percent (device_status->percentage_charge);
 		filename = g_strdup_printf ("gpm-%s-%s-charging", prefix, index_str);
 
-	} else if (device_status->is_discharging) {
+	} else {
 		index_str = gpm_power_get_index_from_percent (device_status->percentage_charge);
 		filename = g_strdup_printf ("gpm-%s-%s", prefix, index_str);
 
-	} else {
-		/* We have a broken battery, not sure what to display here */
-		gpm_debug ("Broken or missing battery...");
-		filename = g_strdup_printf ("gpm-%s-missing", prefix);
 	}
 	return filename;
 }
@@ -1035,7 +1019,7 @@ battery_kind_cache_update (GpmPower		 *power,
 	guint num_discharging = 0;
 	GpmPowerStatus *type_status = &entry->battery_status;
 
-	/* clear old values */
+	/* clear old values (except previous charge rate) */
 	gpm_power_battery_status_set_defaults (type_status);
 
 	/* iterate thru all the devices to handle multiple batteries */
@@ -1071,7 +1055,6 @@ battery_kind_cache_update (GpmPower		 *power,
 		type_status->design_charge += device_status->design_charge;
 		type_status->last_full_charge += device_status->last_full_charge;
 		type_status->current_charge += device_status->current_charge;
-		type_status->charge_rate_smoothed += device_status->charge_rate_smoothed;
 		type_status->charge_rate_raw += device_status->charge_rate_raw;
 		type_status->voltage += device_status->voltage;
 		/* we have to sum this here, in case the device has no rate
@@ -1079,6 +1062,22 @@ battery_kind_cache_update (GpmPower		 *power,
 		type_status->remaining_time += device_status->remaining_time;
 	}
 
+	/* Do an exponentially weighted average, fixes bug #328927 */
+	type_status->charge_rate_smoothed = gpm_power_exp_aver (type_status->charge_rate_previous,
+								type_status->charge_rate_raw,
+								power->priv->exp_ave_factor);
+	
+	/* If the average is healthy, store it as the previous value */
+	if( type_status->charge_rate_smoothed != 0 ) {
+		type_status->charge_rate_previous = type_status->charge_rate_smoothed;
+	} else {
+		gpm_warning("Exponential average of 0 calculated (not okay.):"
+			    " inputs were charge_rate_previous=%d, charge_rate_raw=%d, exp_ave_factor=%d",
+			    type_status->charge_rate_previous,
+			    type_status->charge_rate_raw,
+			    power->priv->exp_ave_factor );
+ 	}
+
 	/* average out the voltage for the global device */
 	if (num_present > 1) {
 		type_status->voltage /= num_present;
@@ -1119,12 +1118,19 @@ battery_kind_cache_update (GpmPower		 *power,
 	    type_status->is_discharging == FALSE &&
 	    type_status->percentage_charge > 0 &&
 	    type_status->percentage_charge < GPM_POWER_MIN_CHARGED_PERCENTAGE) {
-		gboolean on_ac;
-		on_ac = gpm_hal_power_is_on_ac (power->priv->hal_power);
+		GpmAcAdapterState state;
+
+		/* get the ac state */
+		gpm_ac_adapter_get_state (power->priv->ac_adapter, &state);
 		gpm_debug ("Battery is neither charging nor discharging, "
-			   "using ac_adaptor value %i", on_ac);
-		type_status->is_charging = on_ac;
-		type_status->is_discharging = !(on_ac);
+			   "using ac_adaptor value %i", state);
+		if (state == GPM_AC_ADAPTER_PRESENT) {
+			type_status->is_charging = TRUE;
+			type_status->is_discharging = FALSE;
+		} else {
+			type_status->is_charging = FALSE;
+			type_status->is_discharging = TRUE;
+		}		
 	}
 
 	/* We only do the "better" remaining time algorithm if the battery has rate,
@@ -1300,6 +1306,10 @@ power_get_summary_for_battery_kind (GpmPower	 *power,
 	GpmPowerStatus *status;
 	const gchar *type_desc = NULL;
 	gchar *timestring;
+	GpmAcAdapterState state;
+
+	/* get the ac status */
+	gpm_ac_adapter_get_state (power->priv->ac_adapter, &state);
 
 	entry = battery_kind_cache_find (power, battery_kind);
 
@@ -1347,7 +1357,7 @@ power_get_summary_for_battery_kind (GpmPower	 *power,
 						type_desc, status->percentage_charge);
 		}
 
-	} else if (status->is_charging || power->priv->on_ac) {
+	} else if (status->is_charging || state == GPM_AC_ADAPTER_PRESENT) {
 
 		if (status->remaining_time > 60) {
 			g_string_append_printf (summary, _("%s %s until charged (%i%%)\n"),
@@ -1381,6 +1391,10 @@ gpm_power_get_status_summary (GpmPower *power,
 	GString *summary = NULL;
 	gboolean ups_present;
 	GpmPowerStatus status;
+	GpmAcAdapterState state;
+
+	/* get the ac state */
+	gpm_ac_adapter_get_state (power->priv->ac_adapter, &state);
 
 	g_return_val_if_fail (power != NULL, FALSE);
 	g_return_val_if_fail (GPM_IS_POWER (power), FALSE);
@@ -1400,7 +1414,7 @@ gpm_power_get_status_summary (GpmPower *power,
 		   http://bugzilla.gnome.org/show_bug.cgi?id=329027 */
 		summary = g_string_new (_("Computer is running on backup power\n"));
 
-	} else if (power->priv->on_ac) {
+	} else if (state == GPM_AC_ADAPTER_PRESENT) {
 		summary = g_string_new (_("Computer is running on AC power\n"));
 
 	} else {
@@ -1455,94 +1469,6 @@ gpm_power_get_battery_status (GpmPower       *power,
 	return TRUE;
 }
 
-/**
- * gpm_power_set_on_ac:
- * @power: This power class instance
- **/
-static gboolean
-gpm_power_set_on_ac (GpmPower *power,
-		     gboolean  on_ac,
-		     GError  **error)
-{
-	g_return_val_if_fail (power != NULL, FALSE);
-	g_return_val_if_fail (GPM_IS_POWER (power), FALSE);
-
-	if (on_ac != power->priv->on_ac) {
-		power->priv->on_ac = on_ac;
-
-		gpm_debug ("emitting ac-state-changed : %i", on_ac);
-		g_signal_emit (power, signals [AC_STATE_CHANGED], 0, on_ac);
-	}
-	gpm_refcount_add (power->priv->refcount);
-
-	return TRUE;
-}
-
-/**
- * gpm_power_get_on_ac:
- * @power: This power class instance
- **/
-gboolean
-gpm_power_get_on_ac (GpmPower *power,
-		     gboolean *on_ac,
-		     GError  **error)
-{
-	g_return_val_if_fail (power != NULL, FALSE);
-	g_return_val_if_fail (GPM_IS_POWER (power), FALSE);
-
-	if (on_ac) {
-		*on_ac = power->priv->on_ac;
-	}
-
-	return TRUE;
-}
-
-/**
- * gpm_power_set_property:
- **/
-static void
-gpm_power_set_property (GObject	     *object,
-			guint	      prop_id,
-			const GValue *value,
-			GParamSpec   *pspec)
-{
-	GpmPower *power;
-
-	power = GPM_POWER (object);
-
-	switch (prop_id) {
-	case PROP_ON_AC:
-		gpm_power_set_on_ac (power, g_value_get_boolean (value), NULL);
-		break;
-	default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-		break;
-	}
-}
-
-/**
- * gpm_power_get_property:
- **/
-static void
-gpm_power_get_property (GObject    *object,
-			guint       prop_id,
-			GValue     *value,
-			GParamSpec *pspec)
-{
-	GpmPower *power;
-
-	power = GPM_POWER (object);
-
-	switch (prop_id) {
-	case PROP_ON_AC:
-		g_value_set_boolean (value, power->priv->on_ac);
-		break;
-	default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-		break;
-	}
-}
-
 /**
  * gpm_power_class_init:
  **/
@@ -1552,37 +1478,7 @@ gpm_power_class_init (GpmPowerClass *klass)
 	GObjectClass   *object_class = G_OBJECT_CLASS (klass);
 
 	object_class->finalize	   = gpm_power_finalize;
-	object_class->get_property = gpm_power_get_property;
-	object_class->set_property = gpm_power_set_property;
 
-	g_object_class_install_property (object_class,
-					 PROP_ON_AC,
-					 g_param_spec_boolean ("on_ac",
-							       NULL,
-							       NULL,
-							       TRUE,
-							       G_PARAM_READWRITE));
-
-	signals [BUTTON_PRESSED] =
-		g_signal_new ("button-pressed",
-			      G_TYPE_FROM_CLASS (object_class),
-			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (GpmPowerClass, button_pressed),
-			      NULL,
-			      NULL,
-			      gpm_marshal_VOID__STRING_BOOLEAN,
-			      G_TYPE_NONE,
-			      2, G_TYPE_STRING, G_TYPE_BOOLEAN);
-	signals [AC_STATE_CHANGED] =
-		g_signal_new ("ac-power-changed",
-			      G_TYPE_FROM_CLASS (object_class),
-			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (GpmPowerClass, ac_state_changed),
-			      NULL,
-			      NULL,
-			      g_cclosure_marshal_VOID__BOOLEAN,
-			      G_TYPE_NONE,
-			      1, G_TYPE_BOOLEAN);
 	signals [BATTERY_STATUS_CHANGED] =
 		g_signal_new ("battery-status-changed",
 			      G_TYPE_FROM_CLASS (object_class),
@@ -1593,16 +1489,6 @@ gpm_power_class_init (GpmPowerClass *klass)
 			      g_cclosure_marshal_VOID__INT,
 			      G_TYPE_NONE,
 			      1, G_TYPE_INT);
-	signals [BATTERY_REMOVED] =
-		g_signal_new ("battery-removed",
-			      G_TYPE_FROM_CLASS (object_class),
-			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (GpmPowerClass, battery_removed),
-			      NULL,
-			      NULL,
-			      g_cclosure_marshal_VOID__STRING,
-			      G_TYPE_NONE,
-			      1, G_TYPE_STRING);
 	signals [BATTERY_PERHAPS_RECALL] =
 		g_signal_new ("battery-perhaps-recall",
 			      G_TYPE_FROM_CLASS (object_class),
@@ -1618,8 +1504,8 @@ gpm_power_class_init (GpmPowerClass *klass)
 }
 
 /**
- * hal_on_ac_changed_cb:
- * @monitor: The HAL monitor class instance
+ * ac_adaptor_changed_cb:
+ * @battery: The HAL battery class instance
  * @on_ac: If we are on AC power
  * @power: This power class instance
  *
@@ -1627,15 +1513,15 @@ gpm_power_class_init (GpmPowerClass *klass)
  * and kind caches as they have likely changed.
  **/
 static void
-hal_on_ac_changed_cb (GpmHalMonitor *monitor,
-		      gboolean       on_ac,
-		      GpmPower      *power)
+ac_adaptor_changed_cb (GpmAcAdapter *ac_adapter,
+		       GpmAcAdapterState state,
+		       GpmPower *power)
 {
-	gpm_power_set_on_ac (power, on_ac, NULL);
+	/* update the caches */
+	battery_kind_cache_update_all (power);
 
-	if (! on_ac) {
-		battery_kind_cache_update_all (power);
-	}
+	/* add a refcount, as we don't want to trigger a suspend */
+	gpm_refcount_add (power->priv->refcount);
 }
 
 /**
@@ -1712,14 +1598,14 @@ remove_battery (GpmPower    *power,
 }
 
 /**
- * hal_battery_added_cb:
- * @monitor: The HAL monitor class instance
+ * battery_added_cb:
+ * @battery: The HAL battery class instance
  * @udi: The HAL UDI for this device
  * @power: This power class instance
  * Called from HAL...
  **/
 static void
-hal_battery_added_cb (GpmHalMonitor *monitor,
+battery_added_cb (GpmBattery *battery,
 		      const gchar   *udi,
 		      GpmPower      *power)
 {
@@ -1730,38 +1616,34 @@ hal_battery_added_cb (GpmHalMonitor *monitor,
 }
 
 /**
- * hal_battery_removed_cb:
- * @monitor: The HAL monitor class instance
+ * battery_removed_cb:
+ * @battery: The HAL battery class instance
  * @udi: The HAL UDI for this device
  * @power: This power class instance
  * Called from HAL...
  **/
 static void
-hal_battery_removed_cb (GpmHalMonitor *monitor,
-			const gchar   *udi,
-			GpmPower      *power)
+battery_removed_cb (GpmBattery  *battery,
+		    const gchar *udi,
+		    GpmPower    *power)
 {
 	gpm_debug ("Battery Removed: %s", udi);
 
 	remove_battery (power, udi);
 
 	battery_kind_cache_debug_print_all (power);
-
-	/* proxy it */
-	gpm_debug ("emitting battery-removed : %s", udi);
-	g_signal_emit (power, signals [BATTERY_REMOVED], 0, udi);
 }
 
 /**
- * hal_battery_property_modified_cb:
- * @monitor: The HAL monitor class instance
+ * battery_modified_cb:
+ * @battery: The HAL battery class instance
  * @udi: The HAL UDI for this device
  * @key: The HAL key that is modified
  * @power: This power class instance
  * Called from HAL...
  **/
 static void
-hal_battery_property_modified_cb (GpmHalMonitor *monitor,
+battery_modified_cb (GpmBattery *battery,
 				  const gchar   *udi,
 				  const gchar   *key,
 				  gboolean	 finally,
@@ -1803,24 +1685,6 @@ hal_battery_property_modified_cb (GpmHalMonitor *monitor,
 	}
 }
 
-/**
- * hal_button_pressed_cb:
- * @monitor: The HAL monitor class instance
- * @type: the button type, e.g. "power"
- * @power: This power class instance
- * @state: The lid state, where TRUE is open or pressed
- **/
-static void
-hal_button_pressed_cb (GpmHalMonitor *monitor,
-		       const gchar   *type,
-		       gboolean       state,
-		       GpmPower      *power)
-{
-	/* just proxy it */
-	gpm_debug ("emitting button-pressed : %s (%i)", type, state);
-	g_signal_emit (power, signals [BUTTON_PRESSED], 0, type, state);
-}
-
 /**
  * gpm_hash_remove_return:
  * FIXME: there must be a better way to do this
@@ -1930,7 +1794,7 @@ hal_daemon_start_cb (GpmHal     *hal,
 {
 	gpm_hash_new_kind_cache (power);
 	gpm_hash_new_device_cache (power);
-	gpm_hal_monitor_coldplug (power->priv->hal_monitor);
+	gpm_battery_coldplug (power->priv->battery);
 	battery_kind_cache_update_all (power);
 }
 
@@ -1957,38 +1821,41 @@ hal_daemon_stop_cb (GpmHal   *hal,
 static void
 gpm_power_init (GpmPower *power)
 {
-	gboolean on_ac;
-
-	GConfClient *client = gconf_client_get_default ();
+	guint invalid_timeout;
 
 	power->priv = GPM_POWER_GET_PRIVATE (power);
 
+	power->priv->conf = gpm_conf_new ();
+
+	power->priv->ac_adapter = gpm_ac_adapter_new ();
+	g_signal_connect (power->priv->ac_adapter, "ac-adapter-changed",
+			  G_CALLBACK (ac_adaptor_changed_cb), power);
+
 	power->priv->hal = gpm_hal_new ();
 	g_signal_connect (power->priv->hal, "daemon-start",
 			  G_CALLBACK (hal_daemon_start_cb), power);
 	g_signal_connect (power->priv->hal, "daemon-stop",
 			  G_CALLBACK (hal_daemon_stop_cb), power);
 
-	power->priv->hal_monitor = gpm_hal_monitor_new ();
-	g_signal_connect (power->priv->hal_monitor, "button-pressed",
-			  G_CALLBACK (hal_button_pressed_cb), power);
-	g_signal_connect (power->priv->hal_monitor, "ac-power-changed",
-			  G_CALLBACK (hal_on_ac_changed_cb), power);
-	g_signal_connect (power->priv->hal_monitor, "battery-property-modified",
-			  G_CALLBACK (hal_battery_property_modified_cb), power);
-	g_signal_connect (power->priv->hal_monitor, "battery-added",
-			  G_CALLBACK (hal_battery_added_cb), power);
-	g_signal_connect (power->priv->hal_monitor, "battery-removed",
-			  G_CALLBACK (hal_battery_removed_cb), power);
+	power->priv->battery = gpm_battery_new ();
+	g_signal_connect (power->priv->battery, "battery-modified",
+			  G_CALLBACK (battery_modified_cb), power);
+	g_signal_connect (power->priv->battery, "battery-added",
+			  G_CALLBACK (battery_added_cb), power);
+	g_signal_connect (power->priv->battery, "battery-removed",
+			  G_CALLBACK (battery_removed_cb), power);
 
-	power->priv->hal_power = gpm_hal_power_new ();
+	power->priv->hal = gpm_hal_new ();
 
 	power->priv->refcount = gpm_refcount_new ();
 	g_signal_connect (power->priv->refcount, "refcount-zero",
 			  G_CALLBACK (gpm_power_refcount_zero), power);
 	g_signal_connect (power->priv->refcount, "refcount-added",
 			  G_CALLBACK (gpm_power_refcount_added), power);
-	gpm_refcount_set_timeout (power->priv->refcount, GPM_POWER_INVALID_TIMOUT);
+
+	/* we get this from gconf as some machines take longer to settle down */
+	gpm_conf_get_uint (power->priv->conf, GPM_CONF_INVALID_TIMEOUT, &invalid_timeout);
+	gpm_refcount_set_timeout (power->priv->refcount, invalid_timeout);
 
 	/* when we first start, the data might be invalid */
 	gpm_refcount_add (power->priv->refcount);
@@ -1999,13 +1866,8 @@ gpm_power_init (GpmPower *power)
 	gpm_hash_new_kind_cache (power);
 	gpm_hash_new_device_cache (power);
 
-	on_ac = gpm_hal_power_is_on_ac (power->priv->hal_power);
-	gpm_power_set_on_ac (power, on_ac, NULL);
-
-	power->priv->exp_ave_factor = gconf_client_get_int (client,
-							    GPM_PREF_RATE_EXP_AVE_FACTOR,
-							    NULL);
-	g_object_unref (client);
+	gpm_conf_get_uint (power->priv->conf, GPM_CONF_RATE_EXP_AVE_FACTOR,
+			   &power->priv->exp_ave_factor);
 }
 
 /**
@@ -2026,18 +1888,23 @@ gpm_power_finalize (GObject *object)
 	gpm_hash_free_kind_cache (power);
 	gpm_hash_free_device_cache (power);
 
+	g_object_unref (power->priv->conf);
+
 	if (power->priv->hal != NULL) {
 		g_object_unref (power->priv->hal);
 	}
-	if (power->priv->hal_monitor != NULL) {
-		g_object_unref (power->priv->hal_monitor);
+	if (power->priv->battery != NULL) {
+		g_object_unref (power->priv->battery);
 	}
-	if (power->priv->hal_power != NULL) {
-		g_object_unref (power->priv->hal_power);
+	if (power->priv->hal != NULL) {
+		g_object_unref (power->priv->hal);
 	}
 	if (power->priv->refcount != NULL) {
 		g_object_unref (power->priv->refcount);
 	}
+	if (power->priv->ac_adapter != NULL) {
+		g_object_unref (power->priv->ac_adapter);
+	}
 
 	G_OBJECT_CLASS (gpm_power_parent_class)->finalize (object);
 }
@@ -2049,12 +1916,11 @@ gpm_power_finalize (GObject *object)
 GpmPower *
 gpm_power_new (void)
 {
-	if (gpm_power_object) {
-		g_object_ref (gpm_power_object);
+	static GpmPower *power = NULL;
+	if (power != NULL) {
+		g_object_ref (power);
 	} else {
-		gpm_power_object = g_object_new (GPM_TYPE_POWER, NULL);
-		g_object_add_weak_pointer (gpm_power_object,
-					   (gpointer *) &gpm_power_object);
+		power = g_object_new (GPM_TYPE_POWER, NULL);
 	}
-	return GPM_POWER (gpm_power_object);
+	return GPM_POWER (power);
 }
