@@ -25,14 +25,6 @@
 
 G_BEGIN_DECLS
 
-typedef enum {
-	GPM_ICON_POLICY_ALWAYS,
-	GPM_ICON_POLICY_PRESENT,
-	GPM_ICON_POLICY_CHARGE,
-	GPM_ICON_POLICY_CRITICAL,
-	GPM_ICON_POLICY_NEVER
-} GpmIconPolicy;
-
 #define ACTION_SUSPEND			"suspend"
 #define ACTION_SHUTDOWN			"shutdown"
 #define ACTION_HIBERNATE		"hibernate"
